<?php

namespace Drupal\salesforce_mfw\Form;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Configuration form for Salesforce MFW.
 */
class SalesforceMFWConfiguration extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['salesforce_mfw.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'salesforce_mfw_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('salesforce_mfw.settings');

    $form['allowed_paths'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Allowed paths'),
      '#description' => $this->t('Enter one path per line. Use * as a wildcard character.'),
      '#default_value' => $config->get('allowed_paths') ?? '',
    ];

    $form['allow_path_override'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allow Path Override'),
      '#description' => $this->t('Checking this box will bypass allowed paths if there is an active chat interaction.'),
      '#default_value' => $config->get('allow_path_override') ?? FALSE,
    ];

    $form['organization_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Organization id'),
      '#description' => $this->t('Enter the organization of the Salesforce utility'),
      '#default_value' => $config->get('organization_id') ?? '',
      '#required' => TRUE,
    ];

    $form['config_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Config name'),
      '#description' => $this->t('Enter the config name of the Salesforce utility.'),
      '#default_value' => $config->get('config_name') ?? '',
      '#required' => TRUE,
    ];

    $form['site_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Site url'),
      '#description' => $this->t('Enter the site url of the Salesforce utility.'),
      '#default_value' => $config->get('site_url') ?? '',
      '#required' => TRUE,
    ];

    $form['snippet_config'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Snippet config url'),
      '#description' => $this->t('Enter the snippet config url of the Salesforce utility.'),
      '#default_value' => $config->get('snippet_config') ?? '',
      '#required' => TRUE,
    ];

    $form['utility_bootstrap'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Utility bootstrap'),
      '#description' => $this->t('Enter the Salesforce utility bootstrap url.'),
      '#default_value' => $config->get('utility_bootstrap') ?? '',
      '#required' => TRUE,
    ];

    $form['language'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Language'),
      '#description' => $this->t('Enter the language of the utility - I.E - en_US'),
      '#default_value' => $config->get('language') ?? '',

    ];

    $form['allow_token_replacement'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Replace tokens'),
      '#description' => $this->t('Checking this box will allow tokens to be replaced if available in default values'),
      '#default_value' => $config->get('allow_token_replacement') ?? FALSE,
    ];

    // Visible fields table.
    $form['fields_container'] = [
      '#type' => 'details',
      '#title' => $this->t('Pre-Chat fields'),
      '#description' => $this->t('This form provides the ability to add default values to either hidden or visible prechat fields. For more advanced
      set up needs, such as setting a default value of a field via a cookie or a user\'s local storage, check out the <a href="#">API Documentation</a>.'),
      '#open' => TRUE,
    ];

    // Add visible fields to the table.
    $form['fields_container']['fields'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Name'),
        $this->t('Default value'),
        $this->t('Hidden'),
        $this->t('Editable by user'),
        $this->t('Operations'),
      ],
      '#prefix' => '<div id="visible-fields-wrapper">',
      '#suffix' => '</div>',
      '#empty' => $this->t('No fields added yet.'),
    ];

    // Load existing fields from the config.
    $visible_fields = $form_state->get('fields');
    if ($visible_fields === NULL) {
      $visible_fields = $config->get('fields') ?? [];
      $form_state->set('fields', $visible_fields);
    }

    foreach ($visible_fields as $key => $field) {
      $form['fields_container']['fields'][$key]['name'] = [
        '#type' => 'textfield',
        '#default_value' => $field['name'],
        '#required' => TRUE,
      ];

      $form['fields_container']['fields'][$key]['default_value'] = [
        '#type' => 'textfield',
        '#default_value' => array_key_exists('default_value', $field) ? $field['default_value'] : '',
      ];

      $form['fields_container']['fields'][$key]['hidden'] = [
        '#type' => 'checkbox',
        '#default_value' => $field['hidden'] ?? FALSE,
      ];

      $form['fields_container']['fields'][$key]['editable_by_user'] = [
        '#type' => 'checkbox',
        '#default_value' => $field['editable_by_user'] ?? FALSE,
        '#states' => [
          'disabled' => [
            ':input[name="fields[' . $key . '][hidden]"]' => ['checked' => TRUE],
          ],
        ],
      ];

      $form['fields_container']['fields'][$key]['operations'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove'),
        '#name' => 'remove_field_' . $key,
        '#submit' => [[$this, 'removeFieldCallback']],
        '#ajax' => [
          'callback' => [$this, 'updateVisibleFieldsTableAjax'],
          'wrapper' => 'visible-fields-wrapper',
        ],
      ];
    }

    // Add button to add new field.
    $form['fields_container']['add_field'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add field'),
      '#submit' => [[$this, 'addFieldCallback']],
      '#ajax' => [
        'callback' => [$this, 'updateVisibleFieldsTableAjax'],
        'wrapper' => 'visible-fields-wrapper',
      ],
    ];

    // Add the token browser link.
    $form['token_browser'] = [
      '#type' => 'link',
      '#title' => $this->t('Browse available tokens'),
      '#url' => Url::fromRoute('token.tree'),
      '#attributes' => [
        'class' => ['use-ajax'],
        'data-dialog-type' => 'modal',
        'data-dialog-options' => Json::encode(['width' => 700]),
      ],
    ];

    $form['#validate'][] = '::validateFields';

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    // Save existing configuration fields.
    $this->config('salesforce_mfw.settings')
      ->set('allowed_paths', $values['allowed_paths'])
      ->set('allow_path_override', $values['allow_path_override'])
      ->set('organization_id', $values['organization_id'])
      ->set('config_name', $values['config_name'])
      ->set('site_url', $values['site_url'])
      ->set('snippet_config', $values['snippet_config'])
      ->set('utility_bootstrap', $values['utility_bootstrap'])
      ->set('language', $values['language'])
      ->set('allow_token_replacement', $values['allow_token_replacement'])
      ->save();

    // Save visible fields.
    $visible_fields = [];

    if (!empty($values['fields'])) {
      foreach ($values['fields'] as $field) {
        $visible_fields[$field['name']]['name'] = $field['name'];
        $visible_fields[$field['name']]['default_value'] = $field['default_value'];
        $visible_fields[$field['name']]['hidden'] = $field['hidden'];
        $visible_fields[$field['name']]['editable_by_user'] = $field['editable_by_user'];
      }
    }

    $this->config('salesforce_mfw.settings')
      ->set('fields', $visible_fields)
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Custom validation handler to check for duplicate field names.
   */
  public function validateFields(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $field_names = [];
    $triggering_element = $form_state->getTriggeringElement();
    if ($triggering_element['#value']->__toString() !== 'Save configuration') {
      return;
    }

    if (!empty($values['fields'])) {
      foreach ($values['fields'] as $key => $field) {
        if (!empty($field['name'])) {
          if (in_array($field['name'], $field_names)) {
            // Set an error if a duplicate field name is found.
            $form_state->setErrorByName("fields][$key][name", $this->t('The field name %name is duplicated.', ['%name' => $field['name']]));
          }
          else {
            $field_names[] = $field['name'];
          }
        }
      }
    }
  }

  /**
   * Callback to add a new field to visible fields.
   */
  public function addFieldCallback(array &$form, FormStateInterface $form_state) {
    $visible_fields = $form_state->get('fields');
    $visible_fields[] = [
      'name' => '',
      'default_value' => '',
      'editable_by_user' => FALSE,
    ];
    $form_state->set('fields', $visible_fields);
    $form_state->setRebuild(TRUE);
  }

  /**
   * Callback to remove a field from visible fields.
   */
  public function removeFieldCallback(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $field_key = str_replace('remove_field_', '', $triggering_element['#name']);
    $fields = $form_state->get('fields');
    unset($fields[$field_key]);
    $form_state->set('fields', $fields);
    $form_state->setRebuild(TRUE);
  }

  /**
   * Ajax callback to update the visible fields table.
   */
  public function updateVisibleFieldsTableAjax(array &$form, FormStateInterface $form_state) {
    return $form['fields_container']['fields'];
  }

}
